//writeln("begin _build.io -----------------------------------------")

Directory do(    
    nonUnderscoreFolders := method(folders select(f, f name beginsWithSeq("_") not))
)

Log := Object clone do(
    indentLevel := 0
    
    indentEvalOf := method(
		indentLevel = indentLevel + 1
		call sender doMessage(call message argAt(0))
		indentLevel = indentLevel - 1
	)
	
	writeln := method(
		indentLevel repeatTimes(super(write("  ")))
		self := call sender
		resend
    )
    
    write := getSlot("writeln")
)

Module := Object clone do(
    libSearchPaths := List clone
    
    supportedOnPlatform := true

    appendLibSearchPath := method(v, libSearchPaths contains(v) ifNilEval(libSearchPaths append(v)))

    appendLibSearchPath("/usr/lib")
    appendLibSearchPath("/usr/local/lib")
    
    init := method(
		self folder := Directory clone
		self dependsOnLibs := List clone
		self dependsOnFrameworks := List clone
		self dependsOnIncludes := List clone
		self dependsOnLinkOptions := List clone
    )
    
    cc := method(
    	cc := System getenv("CC") ifNilEval(return "cc")
    )

    platform := System platform asLowercase
    
    pathForLib := method(name,
		libname := "lib" .. name .. "."
		path := libSearchPaths detect(path, 
			Directory with(path) files detect(f, f name beginsWithSeq(libname))
		)
		writeln("path = ", path)
		path
    )
        
    dependsOnLib := method(v, 
		dependsOnLibs append(v)
		//Log writeln("appendLib(", v, ")")
    )
    
    dependsOnFramework  := method(v, dependsOnFrameworks append(v))
    dependsOnInclude    := method(v, dependsOnIncludes append(v))
    dependsOnLinkOption := method(v, dependsOnLinkOptions append(v))
    
    with := method(path, 
		module := self clone
		module folder setPath(path)
		module
    )

    make := method(style,
		systemCall("make " .. style ifNilEval(""))
    )
    
    tryMake := method(style,
		trySystemCall("make " .. style ifNilEval(""))
    )
        
    systemCall := method(s,
		if(trySystemCall(s) == 256, System exit(1))    
    )

    trySystemCall := method(s,
		if(folder path != ".", s := "cd " .. folder path .. "; " .. s)
		Log writeln(s .. "\n")
		System system(s) 
    )
        
    modulesInFolder := method(name,
    	folder := Directory clone setPath(name)
    	if(folder exists == false, return List clone)
		subfolders := folder folders
		subfolders selectInPlace(f, f fileNamed("_build.io"))
		subfolders map(f, 
			//writeln("_build.io modulesInFolder")
			module := Lobby doString(f fileNamed("_build.io") contents)
			module folder setPath(f path)
			module
		)	
    )

    linkString := method(
		s := Sequence clone
		dependsOnLibs foreach(libname,
			writeln("libname = ", libname)
			s appendSeq(" -l", libname)
		)
		s appendSeq(dependsOnLinkOptions join(" "), " ")
		s appendSeq(dependsOnFrameworks map(v, "-framework " .. v) join(" "), " ")
		s 
    )
)

Lib := Module clone do(    
    hasLibs := method(
		libsFolder := folder folderNamed("_libs") 
		libsFolder and(libsFolder files size > 0)
    )
    
    build := method(
		writeln("build folder path ", folder path)
		make
		copyHeadersAndLibs
    )
    
    make := method(option,
		writeln("Lib make")
		trySystemCall("cd " .. folder name .. "; make " .. option ifNilEval(""))
    )

    createHeadersAndLibsFolders := method(
		folder createSubdirectory("_libs")
		folder createSubdirectory("_includes")
    )
    
    copyHeadersAndLibs := method(
		createHeadersAndLibsFolders
		name := folder name
		trySystemCall("cp " .. name .. "/lib*.a _libs")
		trySystemCall("cp " .. name .. "/*/lib*.a _libs")
		trySystemCall("cp " .. name .. "/.libs/lib*.a _libs")
		trySystemCall("cp " .. name .. "/*/.libs/lib*.a _libs")
		trySystemCall("cp " .. name .. "/*/*/.libs/lib*.a _libs")
	
		trySystemCall("cp -R " .. name .. "/include/* _includes")
		trySystemCall("cp " .. name .. "/*/*.[h] _includes")
		trySystemCall("cp " .. name .. "/*.h _includes")
		trySystemCall("ranlib _libs/lib*.a")
    )
    
    removeHeadersAndLibs := method(
		trySystemCall("rm -rf _libs")
		trySystemCall("rm -rf _includes")
		//folder folderNamed("_libs") ?remove
		//folder folderNamed("_includes") ?remove
    )
    
    clean := method(
		removeHeadersAndLibs
		make("clean")
		File with(folder name .. "/build/unix/libtool") remove
    )
    
    configure := method(
		trySystemCall("chmod +x " .. folder name .. "/configure")
		trySystemCall("cd " .. folder name .. "; ./configure") 
    )
    
    isBuilt := method(hasLibs)
    
    buildIfNeeded := method(
		//isBuilt := method(hasLibs)
		if(isBuilt) then(
			Log writeln(folder name, " [done]")
		) else(
			Log writeln(folder name)
			Log indentEvalOf(configure; build)
		)
		
		//trySystemCall("ranlib _libs/lib*.a")
		appendLibSearchPath(Path with(folder path, "_libs"))
    )
    
    linkString := method(
		s := folder folderNamed("_libs") files map(f, 
			"-l" .. (f name asMutable removePrefix("lib") removeSuffix(".a"))
		) join(" ")
		s .. super(linkString)
    )
)

Binding := Module clone do(
    libName := method("libIo" .. folder name ..  ".a")
    libFile := method(folder fileNamed(libName))
    objsFolder := method(self objsFolder := folder createSubdirectory("_objs"))
    cFiles := method(folder filesWithExtension("c") appendSeq(folder filesWithExtension("m")))
    lastCFileChange := method(cFiles sortInPlace(lastDataChangeDate) last lastDataChangeDate)
    lastIoFileChange := method(ioFiles sortInPlace(lastDataChangeDate) last lastDataChangeDate)
    lastLibFileChange := method(libFile lastDataChangeDate)
    
    needsToBuildLib := method(
        (libFile == nil) or(libFile and(lastLibFileChange < lastCFileChange)) or(
        ioCodeFolder and(ioFiles size > 0) and(lastLibFileChange < lastIoFileChange))
    )
    
    buildLib := method(
		if(needsToBuildLib,
			objNames := objsFolder filesWithExtension("o") map(f, "_objs/" .. f name)
			objNames := (" " join(objNames))
			systemCall("ar rcu " .. libName .. " " .. objNames)
			systemCall("ranlib " .. libName)
			//systemCall("gcc -bundle -o lib" .. self name .. ".bundle " .. objNames)
		)    
    )
    
    build := method(options,
		appendLibSearchPath(folder path)
		if(needsToBuildLib == nil, 
			Log writeln(folder name, " [done]")
			systemCall("ranlib " .. libName)
			return
		)
		generateInitFile
	
		Log writeln(folder name)
		Log indentEvalOf(
			options := options ifNilEval("")
			cFiles foreach(f, 
			obj := f name replaceSeq(".c", ".o") replaceSeq(".m", ".o") 
			objFile := objsFolder fileNamed(obj)
			if((objFile == nil) or(objFile lastDataChangeDate < f lastDataChangeDate)) then(
				//s := "cc " .. options .. " -I. -fPIC -c -o _objs/" .. obj .. " " .. f name
				s := cc .. " " .. options .. " " .. includePaths map(i, "-I" .. Path with("../../", i)) join(" ") .. " -I. -c -o _objs/" .. obj .. " " .. f name
				systemCall(s)
			)
			)
			buildLib
		)
    )
    
    objectFiles := method(
		if(objsFolder, return objsFolder filesWithExtension("o"))
		List clone
    )
    
    clean := method(
		objectFiles foreach(f, f remove)
		folder folderNamed("_objs") ?remove
		self removeSlot("objsFolder")
		libFile ?remove
		folder fileNamed(initFileName) ?remove
		Log writeln(folder name, " clean")
    )
    
    ioCodeFolder := method(folder folderNamed("_ioCode"))
    ioFiles      := method(ioCodeFolder filesWithExtension("io"))
    initFileName := method("Io" .. folder name .. "Init.c")
    
    generateInitFile := method(
		if(folder filesWithExtension("m") size != 0, return)
		initFile := folder createFileNamed(initFileName)
		initFile remove open
		initFile write("#include \"IoState.h\"\n") 
		initFile write("#include \"IoObject.h\"\n\n") 
		
		iocFiles := folder files select(f, f name beginsWithSeq("Io") and(f name endsWithSeq(".c")) and(f name containsSeq("Init") not) and(f name containsSeq("_") not))
		extraFiles := folder files select(f, f name beginsWithSeq("Io") and(f name endsWithSeq(".c")) and(f name containsSeq("Init") not) and(f name containsSeq("_")))
	
		orderedFiles := List clone appendSeq(iocFiles)
		
		iocFiles foreach(f,
			//writeln("checking for depends on ", f path)
			d := f open readLines detect(line, line containsSeq("docDependsOn"))
			f close
			
			if(d) then(
				prerequisitName := "Io" .. d afterSeq("(\"") beforeSeq("\")") .. ".c"
				writeln(">>> ", f path, " depends on ", prerequisitName)
				prerequisit := orderedFiles detect(of, of name == prerequisitName )
				orderedFiles foreach(v, v name println)
				orderedFiles remove(f) 
				orderedFiles insertAfter(f, prerequisit)
				orderedFiles foreach(v, v name println)
			)
		)
		
		iocFiles = orderedFiles
		
		iocFiles foreach(f, 
			initFile write("IoObject *" .. f name fileName .. "_proto(void *state);\n")
		)
	
		extraFiles foreach(f,
			initFile write("void " .. f name fileName .. "Init(void *state, void *context);\n")
		)
		
		initFile write("\nvoid Io" .. folder name .. "Init(IoState *self, IoObject *context)\n")
		initFile write("{\n")
		
		iocFiles foreach(f, 
			initFile write("	IoObject_setSlot_to_(context, SIOSYMBOL(\"" .. f name fileName asMutable removePrefix("Io") .. "\"), " .. f name fileName .. "_proto(self));\n")
		)
		
		extraFiles foreach(f,
			initFile write("	" .. f name fileName .. "Init(self, context);\n")
		)
	
		if(ioCodeFolder, ioCodeFolder filesWithExtension("io") sortInPlace(name) foreach(f, initFile write(codeForIoFile(f))))
		initFile write("}\n")
		initFile close
    )

    codeForIoFile := method(f,
		code := Sequence clone
    	if (f size > 0,
			code appendSeq("  {\n  char *s = ")
			code appendSeq(f contents splitNoEmpties("\n") map(line, "\"" .. line escape .. "\\n\"") join("\n\t"))
			code appendSeq(";\n  IoState_on_doCString_withLabel_(self, context, s, \"" .. f name .. "\");\n")
			code appendSeq("  }\n\n")
		)
		code
    )
    
    includePaths := method(
		libIncludes     := Directory with("libs") nonUnderscoreFolders map(f, Path with(f path, "_includes"))
		bindingIncludes := Directory with("bindings") nonUnderscoreFolders map(f, f path)
		vmIncludes      := list("/vm/_include", "vm/_include/base", "vm/_include/SkipDB")
		vmIncludes appendSeq(libIncludes) appendSeq(bindingIncludes) 
    )
    
    linkString := method("-lIo" .. folder name )
    
    generateDocs := method(
    	systemCall("../../binaries/ioDesktop ../../vm/_docgen/DocsExtractor.io .")
    )
    
    cleanDocs := method(
    	systemCall("rm _ioCode/zzz_docs.io | true")
    )
)


Project := Module clone do(
	newSlot("didBuildIo", false)
	newSlot("didBuildIoServer", false)
	newSlot("didBuildIoDesktop", false)
	
    libs     := method(
    	modulesInFolder("libs")
    )
    
    bindings := method(
        self bindings := modulesInFolder("bindings") sortInPlaceBy(block(x, y, x folder name < y folder name))
		bindings selectInPlace(b, b supportedOnPlatform(platform))
		bindings
    )

    build := method(
		writeln("Project build")
		buildLibs
		buildBindings
		buildDesktop
    )
    
    buildLibs := method(
		Log writeln("libs")
    	if(libs size == 0, writeln("no libs to build"); return)
		Log indentEvalOf(libs foreach(lib, lib buildIfNeeded))
		systemCall("rm -rf libs/_libs")
		systemCall("mkdir -p libs/_libs")
		systemCall("cp libs/*/_libs/*.a libs/_libs")
    )
    
    buildBindings := method(
		Log writeln("bindings")
    	if(bindings size == 0, writeln("no libs to build"); return)
		bindings foreach(generateInitFile)
		Log indentEvalOf(
			bindings foreach(build(options .. " " .. includesString))
		)
		systemCall("rm -rf bindings/_libs")
		systemCall("mkdir -p bindings/_libs")
		systemCall("cp bindings/*/*.a bindings/_libs")
		trySystemCall("ranlib bindings/_libs/lib*.a")
		self
    )
        
    runUnitTests := method(
		Log writeln("bindings unit tests")
		systemCall("cd vm; make test")
		Log indentEvalOf(
			bindings foreach(binding, 
				path := Path with(binding folder path, "_unitTests/run.io")
				if(File clone setPath(path) exists,
					systemCall("./binaries/ioServer " .. path)
				)
			)
		)
		self
    )
    
    //options := "-static -Os -g -Wall -DSANE_POPEN -DIOBINDINGS"
    options := "-Os -g -Wall -DSANE_POPEN -DIOBINDINGS"

    includePaths := method(
		libIncludes     := libs map(l, Path with(l folder path, "_includes"))
		bindingIncludes := bindings map(b, b folder path)
		vmIncludes := list("vm/_include", "vm/_include/base", "vm/_include/SkipDB")
		vmIncludes appendSeq(libIncludes) appendSeq(bindingIncludes)   
		vmIncludes
    )
    
    includesString := method(
    	includePaths map(s, "-I../../" .. s) join(" ")
    )
    
    linkString := method(
		s := Sequence clone
		s appendSeq("-Lbindings/_libs -Llibs/_libs -Lvm/_libs -lIoVM ") // -Lvm/base -lIoBase -Lvm/SkipDB/_libs -lskipdb"
		if(platform != "darwin", s appendSeq("-lm "))
		if(list("linux", "syllable", "sunos") contains(platform), s appendSeq("-ldl "))
		s appendSeq(bindings map(b, b linkString) join(" "))
		
		m := Map clone
		bindings foreach(b, b dependsOnLibs foreach(libname, m atPut(libname, nil)))
		m keys foreach(key, s appendSeq(" -l", key))
		
		m := Map clone
		bindings foreach(b, b dependsOnFrameworks foreach(libname, m atPut(libname, nil)))
		m keys foreach(key, s appendSeq(" -framework ", key))
		s
    )
    
    linkIncludesString := method(
		vmIncludes := list("vm/_include", "vm/_include/base", "vm/_include/SkipDB")
		vmIncludes map(s, "-I" .. s) join(" ")
    )
    
    generateBindingsInitFile := method(
		f := File with("IoBindingsInit.c") remove open
		f write("\n")
		bindings foreach(b, f write("void Io" .. b folder name .. "Init(void *self, void *context);\n"))
		f write("\n")
		f write("void IoBindingsInit(void *self, void *context)\n{\n")
		bindings foreach(b, f write("	Io" .. b folder name .. "Init(self, context);\n"))
		f write("}\n\n")
		f close
    )
    
    buildDesktop := method(
		systemCall("cp vm/io binaries")
    	if(libs size == 0, writeln("no libs - skipping build of ioDesktop"); return)
		trySystemCall("ranlib libs/_libs/lib*.a")
		trySystemCall("ranlib bindings/_libs/lib*.a")
		buildExe("binaries/ioDesktop")
		buildServer
		writeln("--- build successfully completed ---")
    )

    buildServer := method(
    	if(libs size == 0, writeln("no libs - unable to build ioServer"); return)
		serverBindingNames := list( "AAVector", "Blowfish", "Curses", "Fnmatch", "MD5", "Regex", "SGML", "SHA1", "SkipDB", "Sockets", "SQLite", "SQLite3", "Syslog", "User")
		fullBindings := bindings 
		bindings = bindings select(b, b folder name in(serverBindingNames))
		buildExe("binaries/ioServer")
		bindings = fullBindings
		generateBindingsInitFile
    )
        
    buildExe := method(exeName,
		generateBindingsInitFile
		libPathsString := Sequence clone
		Module libSearchPaths foreach(path, libPathsString appendSeq("-L", path, " "))
		systemCall(cc .. " " .. options .. " " .. linkIncludesString .. " -c -o IoBindingsInit.o IoBindingsInit.c")
		systemCall(cc .. " " .. options .. " " .. linkIncludesString .. " -c -o vm/main.o vm/main.c")
		s := cc .. " " .. options .. " -o " .. exeName .. " vm/main.o IoBindingsInit.o "  .. linkIncludesString .. " " .. libPathsString .. linkString  
		systemCall(s)
    )

    cleanLibs := method(
		writeln("--- Project cleanLibs ---")
		libs foreach(clean)
		systemCall("rm -rf libs/_libs")
    )

    cleanBindings := method(
		writeln("--- Project cleanBindings ---")
		bindings foreach(clean)
		systemCall("rm -rf bindings/_libs")
    )

    clean := method(
		writeln("--- Project clean ---")
		cleanLibs
		cleanBindings
		trySystemCall("rm binaries/*")
		trySystemCall("rm -rf projects/*/build")
		trySystemCall("rm IoBindingsInit.*")
   )

    docs := method(
		writeln("--- Project generate embedded docs ---")
		systemCall("cd vm; make docs")
		bindings foreach(generateDocs)
    )

    cleanDocs := method(
		writeln("--- Project clean embedded docs ---")
		systemCall("cd vm; make cleanDocs")
		bindings foreach(cleanDocs)
    )
                
    configure := method(
		libs foreach(configure)
    )
)

Directory setCurrentWorkingDirectory(launchPath)

//if(args at(1), Project clone doString(args at(1)), Project clone buildBindings buildDesktop)
if(args at(1), Project clone doString(args at(1)), Project clone build)

System exit(0)


default := method(
	key := call message argAt(0) name
	call sender getSlot(key) ifNil(call sender setSlot(key, call sender doMessage(call message argAt(1))))
)

