/*  Copyright (c) 2004, Steve Dekorte
 *  All rights reserved.
 */

/*#include "base/Common.h"*/

#ifndef NUMBERARRAY
#define NUMBERARRAY

#define NUM_TYPE float

typedef struct
{
  long size;
  NUM_TYPE *values;
} Vector;

Vector *Vector_new(void);
Vector *Vector_clone(Vector *self);
Vector *Vector_subarray_(Vector *self, int localStart, int length);
void Vector_free(Vector *self);

void Vector_copy_(Vector *self, Vector *other);

int Vector_size(Vector *self);
void Vector_sizeTo_(Vector *self, int size);

NUM_TYPE Vector_at_(Vector *self, int i);
void Vector_at_put_(Vector *self, int i, NUM_TYPE v);

/* ------------------------------------------------------------------------ */
void Vector_addScalar_(Vector *self, NUM_TYPE v);
void Vector_subtractScalar_(Vector *self, NUM_TYPE v);
void Vector_multiplyScalar_(Vector *self, NUM_TYPE v);

/* ----------------------------------------------------------- */
int Vector_addArray_(Vector *self, Vector *other);
int Vector_subtractArray_(Vector *self, Vector *other);
int Vector_multiplyArray_(Vector *self, Vector *other);
NUM_TYPE Vector_dotProduct_(Vector *self, Vector *other);
void Vector_square(Vector *self);
void Vector_normalize(Vector *self);
void Vector_absolute(Vector *self);

void Vector_log(Vector *self);
void Vector_log10(Vector *self);
void Vector_pow(Vector *self, int n);
void Vector_random(Vector *self, float min, float max);

void Vector_sort(Vector *self);
void Vector_reverseSort(Vector *self);

/* ----------------------------------------------------------- */
NUM_TYPE Vector_sum(Vector *self);
NUM_TYPE Vector_min(Vector *self);
NUM_TYPE Vector_max(Vector *self);
NUM_TYPE Vector_mean(Vector *self);
NUM_TYPE Vector_meanSquare(Vector *self);
NUM_TYPE Vector_rootMeanSquare(Vector *self);

void Vector_setMin_(Vector *self, NUM_TYPE v);
void Vector_setMax_(Vector *self, NUM_TYPE v);
void Vector_set_(Vector *self, NUM_TYPE v);

void Vector_sin(Vector *self, 
  int start, int length, 
  NUM_TYPE amplitude, NUM_TYPE offset, NUM_TYPE wavelength);
  
#endif
