/*  copyright: Steve Dekorte, 2002
 *  All rights reserved. See _BSDLicense.txt.
 *
 *  This is an object wrapper for libpng. 
 *
 *  It's currently only useful for decoding a libpng file
 *  to a byte array and providing info about the
 *  format, width, height and # of components.
 *
 *  It's possible to use an external ByteArray for loading by calling
 *  PNGImage_setExternalByteArray_(), but be carefull not to use the 
 *  PNGImage instance after freeing it's external byte array.
 */
 
#ifndef PNGIMAGE_DEFINED
#define PNGIMAGE_DEFINED 1

#include <png.h>
#include <ByteArray.h>

typedef struct
{
  char *path;
  int width;
  int height;
  int components;
  ByteArray *byteArray;
  unsigned char ownsBuffer;
  char *error;
} PNGImage;

PNGImage *PNGImage_new(void);
PNGImage *PNGImage_newWithPath_(char *fname);
void PNGImage_free(PNGImage *self);

void PNGImage_path_(PNGImage *self, const char *path);
char *PNGImage_path(PNGImage *self);

void PNGImage_error_(PNGImage *self, const char *path);
char *PNGImage_error(PNGImage *self);

void PNGImage_load(PNGImage *self);
void PNGImage_save(PNGImage *self);

int PNGImage_width(PNGImage *self);
int PNGImage_height(PNGImage *self);

void PNGImage_width_(PNGImage *self, int w);
void PNGImage_height_(PNGImage *self, int h);
void PNGImage_components_(PNGImage *self, int h);

int PNGImage_pngColorType(PNGImage *self); /* private */

unsigned char PNGImage_isRGB8(PNGImage *self);
unsigned char PNGImage_isRGBA8(PNGImage *self);
unsigned char PNGImage_isL8(PNGImage *self);
unsigned char PNGImage_isLA8(PNGImage *self);
int PNGImage_components(PNGImage *self);

ByteArray *PNGImage_byteArray(PNGImage *self);

/* Whomever calls PNGImage_setExternalByteArray_ is responsible 
 * for freeing "ba" and for making sure that this
 * PNGImage is not still using it after it is freed.
 */
void PNGImage_setExternalByteArray_(PNGImage *self, ByteArray *ba); 

#endif 
