/*#io
Box ioDoc(
		docCopyright("Steve Dekorte", 2002)
		docLicense("BSD revised")
*/

#include "IoBox_gl.h"
#include "IoState.h"
#include "IoNumber.h"
#include "IoVector_gl.h"
#include "IoOpenGL.h"
#include <math.h>

#define DATA(self) ((IoBoxData *)IoObject_dataPointer(self))

#define GLVERTEX2 glVertex2f
#define GLRECT glRectf

IoObject *IoBox_drawAsRect(IoBox *self, IoObject *locals, IoMessage *m)
{ 
  NUM_TYPE x, y, z, w, h, d;
  IoVector_rawGetXYZ(DATA(self)->origin, &x, &y, &z);
  IoVector_rawGetXYZ(DATA(self)->size, &w, &h, &d);
  GLRECT(x, y, x+w, y+h);
  return self; 
}

IoObject *IoBox_drawAsRectOutline(IoBox *self, IoObject *locals, IoMessage *m)
{ 
  NUM_TYPE x, y, z, w, h, d;
  IoVector_rawGetXYZ(DATA(self)->origin, &x, &y, &z);
  IoVector_rawGetXYZ(DATA(self)->size, &w, &h, &d);
  glBegin(GL_LINE_LOOP);
  GLVERTEX2(x, y);
  GLVERTEX2(x, y+h);
  GLVERTEX2(x+w, y+h);
  GLVERTEX2(x+w, y);
  glEnd();
  return self; 
}

IoObject *IoBox_scissor(IoBox *self, IoObject *locals, IoMessage *m)
{ 
  NUM_TYPE x, y, z, w, h, d;
  IoVector_rawGetXYZ(DATA(self)->origin, &x, &y, &z);
  IoVector_rawGetXYZ(DATA(self)->size, &w, &h, &d);
  printf("Scissor: %i %i, %i %i\n", (int)x, (int)y, (int)w, (int)h);
  glScissor((GLint)x, (GLint)y, (GLsizei)w, (GLsizei)h);
  return self; 
}

/*
IoObject *IoBox_scissorToUnion(IoBox *self, IoObject *locals, IoMessage *m)
{ 
  IoBox *other = IoMessage_locals_boxArgAt_(m, locals, 0);
  double x1 = self->origin->x->n;
  double y1 = self->origin->y->n;
  double x2 = x1 + self->size->x->n;
  double y2 = y1 + self->size->y->n;

  double ox1 = other->origin->x->n;
  double oy1 = other->origin->y->n;
  double ox2 = ox1 + other->size->x->n;
  double oy2 = oy1 + other->size->y->n;

  double ux1 = x1 > ox1 ? x1 : ox1;
  double uy1 = y1 > oy1 ? y1 : oy1;
  double ux2 = x2 < ox2 ? x2 : ox2;
  double uy2 = y2 < oy2 ? y2 : oy2;
  
  double uw = ux2 - ux1;
  double uh = uy2 - uy1;
  glScissor(ux1, uy1, uw, uh);
  return self; 
}

IoObject *IoBox_getScissor(IoBox *self, IoObject *locals, IoMessage *m)
{ 
  double v[4];
  glGetDoublev(GL_SCISSOR_BOX, v);
  self->origin->x->n = v[0];
  self->origin->y->n = v[1];
  self->size->x->n = v[2] - v[0];
  self->size->x->n = v[3] - v[1];  
  return self; 
}
*/

IoObject *IoBox_glProject(IoBox *self, IoObject *locals, IoMessage *m)
{ 
  IoVector_glProject(DATA(self)->origin, locals, m);
  /* TODO: need to scale */
  return self; 
}

IoObject *IoBox_glUnproject(IoBox *self, IoObject *locals, IoMessage *m)
{ 
  IoVector_glUnproject(DATA(self)->origin, locals, m);
  /* TODO: need to scale */
  return self; 
}

void IoBox_glInit(IoState *state, IoObject *context) 
{
  IoObject *self = IoState_protoWithInitFunction_(state, IoBox_proto);
  {
  IoMethodTable methodTable[] = {
  {"drawAsRect", IoBox_drawAsRect},
  {"drawAsRectOutline", IoBox_drawAsRectOutline},
  {"scissor", IoBox_scissor},
  /*
  {"scissorToUnion", IoBox_scissorToUnion},
  {"getScissor", IoBox_getScissor},
  */
  {"glProject", IoBox_glProject},
  {"glUnproject", IoBox_glUnproject},
  {NULL, NULL},
  };
  IoObject_addMethodTable_(self, methodTable);
  }
}

