#
# Fountain.io 1.0 - 2005 Mike Ausitn
#

Actor := Object clone do (

  appendProto( OpenGL )

  init := method (
    self subnodes := List clone
    self position := Vector clone
    self velocity := Vector clone
    self age := 0

    reset()
  )

  addChildNode := method ( node,
    subnodes add( node )
    self
  )

  setVelocity := method ( x, y, z,
    velocity set( x, y, z )
    self
  )

  reset := method (
    position set( 0, 0, 0 )
    velocity set( Random value / 200 - .0025,
                  Random value / 30  + .0025,
                  Random value / 200 - .0025 )
    self age = 0 
  )

  advance := method (
    velocity setY( velocity y - .001 )
    position += velocity

    if( position y < 0 ) then (
      if( velocity y abs < .001 ) then (
        reset()
      ) else (
        velocity setY( velocity y * -0.25 )
      )
    )

    self age = age + 1

    subnodes foreach ( node,
      node advance()
    )
  )

  display := method ( displayList,
    subnodes foreach( node,
      glColor4d( 1.0, 1 / (node age / 20), 0, 1 / (node age / 30) );
      glVertex3d( node position x, node position y, node position z )
    )
  )

)

// -------------------------------------------------------------------------- //

Particle := SceneNode clone do (
  AXIS_Y := 1
  LESS_THAN := 0

  init := method (
    setPosition( 0, 0, 0 )
    setVelocity( Random value / 200 - .0025,
                 Random value / 30  + .0025,
                 Random value / 200 - .0025 )
  )

  addCallback( AXIS_Y, LESS_THAN, 0,
    //setPosition( 0, 0, 0 ) setVelocity( .001, 0.03, 0 )
    setPosition( 0, 0, 0 ) setVelocity( Random value / 200 - .0025,
                                        Random value / 30  + .0025,
                                        Random value / 200 - .0025 )
  )

  // 1 / (AGE * 0.05 + 0)
  //setRedFunc( ONE_OVER, AGE, 0.05, 0 )
)

Screen := Object clone do (

  appendProto( OpenGL )

  timer := method (
    scene advance()
    glutPostRedisplay()
    glutTimerFunc( 10, 10 )
  )

  display := method (
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )

    glLoadIdentity()
    glTranslated( 0, -.3, -1 )

    glBegin( GL_POINTS )
      scene display()
    glEnd()

    glutSwapBuffers()
  )

  reshape := method ( width, height,
    glMatrixMode( GL_PROJECTION )
      glLoadIdentity()
      gluPerspective( 45, width / height, 1, 100 )
    glMatrixMode( GL_MODELVIEW )
    glViewport( 0, 0, width, height )
  )

  init := method (
    glutInit()
    glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH )
    glutInitWindowPosition( 200, 100 )
    glutInitWindowSize( 200, 300 )
    glutCreateWindow( "Particles" )

    glutDisplayFunc( display )
    glutReshapeFunc( reshape )
    glutEventTarget( self )

    glEnable( GL_BLEND ); glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    glEnable( GL_POINT_SMOOTH ); glPointSize( 3 )

    glNewList( 100, GL_COMPILE )
      glVertex3d( 0, 0, 0 )
    glEndList()

    //self scene := Actor clone setVelocity( 0, 0, 0 )
    self scene := SceneNode clone

    for( i, 1, 100,
      //scene addChildNode( Actor clone )
      scene addChildNode( Particle clone )
    )
  )

  run := method (
    glutTimerFunc( 10, 10 )
    glutMainLoop()
  )

)

Screen clone run
