// game.io - A simple Openprogram
// (C) 2002 Mike Austin
// Some mods by Steve Dekorte

appendProto(OpenGL)

leftButton := 0
rightButton := 0
upButton := 0

// An base for all game objects
GameObject := Object clone do(
  init := method( x, y,
    self angle := 0
    self x := if( x, x, 0 )
    self y := if( y, y, 0 )
    self dx := Random value / 100
    self dy := Random value / 100
    self
  )
  render := method(
    glPushMatrix
      glTranslated(x, y, 0 )
      glRotated(angle, 0, 0, 1 )
      self renderObject
    glPopMatrix
  )
  rotateLeft := method(angle = angle + 5)
  rotateRight := method(angle = angle - 5)
  thrustForward := method(
    a := self angle * (3.1419 / 180)
    dx = dx - (a sin / 100)
    dy = dy - (a cos / 100)
  )
  update := method(
    x = x + dx
    y = y + dy
  )
)

// Define a ship object
//Ship := Object clone do(
Ship := GameObject clone do(
  renderObject := method(
    glColor4d( 1, 1, 1, 1 )
    glScaled(5, 5, 0)
    glBegin( GL_LINE_LOOP )
       glVertex3d(  0,  3, 0 )
       glVertex3d( -2, -2, 0 )
       glVertex3d(  2, -2, 0 )
    glEnd
  )
)

// Create a Ship object
ship := Ship clone

// Define application object
GLGameApp := Object clone do(
  Screen := Object clone do(
    width := 0
    height := 0
    // Create a couple models
    objs := List clone append( ship, Ship clone( 10, 10 ) )

    //Define reshape callback
    reshape := method( w, h,
      self width := w
      self height := h
      glViewport(0,0,w,h)
      glMatrixMode(GL_PROJECTION)
      glLoadIdentity
      gluOrtho2D(-1,-1,1,1)
      glMatrixMode(GL_MODELVIEW)
      glLoadIdentity
    )

    // Define display callback
    display := method(
      glClear(GL_COLOR_BUFFER_BIT)
      glLoadIdentity
      glTranslated(-1, -1, 0)
      glScaled(2/self width, 2/height, 0)

      // Iterate through the object list
      objs foreach( i, obj,
        obj render
      )

      glFlush
      glutSwapBuffers
    )

    // Define keyboard callback
    keyboard := method( key, x, y,
      if( key asCharacter == "4", leftButton = 1 )
      if( key asCharacter == "6", rightButton = 1 )
      if( key asCharacter == "8", upButton = 1 )

      //glutPostRedisplay
    )

    keyboardUp := method( key, x, y,
      if( key asCharacter == "4", leftButton = 0 )
      if( key asCharacter == "6", rightButton = 0 )
      if( key asCharacter == "8", upButton = 0 )
    )

    timer := method( v,
      glutTimerFunc(10, 0)
      if( leftButton == 1, ship rotateLeft )
      if( rightButton == 1, ship rotateRight )
      if( upButton == 1, ship thrustForward )

      objs foreach(i, obj, 
        obj update
        if (obj x > self width, obj x := 0)
        if (obj y > height, obj y := 0)
        if (obj x < 0, obj x := self width)
        if (obj y < 0, obj y := height)
        obj update
      )
      //self display
      self glutPostRedisplay
    )
  )

  // Define application run method
  run := method(
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA );
    glutInitWindowPosition( 200, 100 )
    glutInitWindowSize( 512, 512 )
    glutInit; glutCreateWindow( "Io Rocks" )
    glutEventTarget( Screen )
    glutReshapeFunc; glutDisplayFunc
    glutKeyboardFunc; glutKeyboardUpFunc
    glutTimerFunc(0, 0)
    glClearColor( 0, 0, 0, 1 )
    glutIgnoreKeyRepeat( 1 )

    glEnable( GL_BLEND )
    glEnable( GL_LINE_SMOOTH )
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA )
    glLineWidth( 1.5 )

    glutMainLoop
  )
)

write( "4 := left  6 := right  8 := thrust\n" )

// Create an GLGameApp and run it
GLGameApp run
