// Player

doFile("CycleModel.yindo")

Player := Object clone do(
    rotY := 0,
    xpos := 0,
    zpos := 0,
    speed := .01,
    walls := ,
    glanceAngle := 0,
    isAI := 0, 
    dead := 0, 
    wallHeight := .05,
    rotZ := 0, 
    normalSpeed := .007,
    red := 1.0,
    green := 1.0,
    blue := 1.0

    init := method(
      self prototype := Player
      self walls := List clone
      chooseRandomPosition
      newWall
      r := random
      g := random
      b := random
      if (r > g and r > b, g = g * .5 b = b * .5 r := 1 )
      if (g > r and g > b, r = r * .5 b = b * .5  g := 1 )
      if (b > r and b > g, r = r * .5 g = g * .5  b := 1 )
      setColor(r, g, b)
    )

    setColor := method(r,g,b, 
	self red := r 
	self green := g 
	self blue := b
    )

    walls := method(walls)
    xyz := method(return xpos, 0, zpos; )

    isAI := methood(aBool, isAI := 1; self)

    newWall := method(
	curWall := currentWall
	if (curWall, curWall x2z2(xpos, zpos))
	wall := Wall clone
	wall player(self)
	wall x1z1(xpos, zpos)
	wall x2z2(xpos, zpos)
	walls append(wall)
	WallManager addWall(wall, self rotY + 90)
	wall
    )

    currentWall := method(walls last)

    draw := method(
      drawWalls
      if (dead == 1, return )
      glPushMatrix
       dx, dy, dz := dxdydzNormalized
      glTranslate(xpos + (dx / 100), 0, zpos + (dz / 100))
      glRotate(rotY, 0, 1, 0)
      glRotate(rotZ, 0, 0, 1)
      CycleModel draw
      glPopMatrix
    )

    drawWalls := method(
       walls foreach(wall, wall draw( red, green, blue))
    )

    keyboard := method(key, x, y,
       key := strchar(key)

      //print("player key := "..tostring(key))           

      if (      key == 'c' ) then (applyBrake)
	elseif (key == 'z' ) then (makeRightTurn)
	elseif (key == 'x' ) then (makeLeftTurn) 
	elseif (key == ',' ) then (lookLeft)
	elseif (key == '.' ) then (lookRight)
      )
    )

    lookLeft := method(glance(90))
    lookRight := method(glance(-90))

    glance := method(v,  
	if (glanceAngle == v, glanceAngle = 0, glanceAngle = v)
    )

    applyBrake := method(
	s :=  normalSpeed / 5
	speed = s + (speed - s) * .95
    )

    chooseRandomPosition := method(
       xpos = random * 4 - 3
       zpos = random * 4 - 3
    )

    makeRandomTurn := method(
	if (random < .5,  makeLeftTurn,  makeRightTurn)
    )

    makeLeftTurn := method(
      self rotY := floor(self rotY - 90)
      if (self rotY < 0 ) then ( self rotY := 360 + self rotY )
      //self rotZ := 45
      self newWall
    )

    makeRightTurn := method(
      self rotY := floor(self rotY + 90)
      if (self rotY > 360 ) then ( self rotY := self rotY - 360 )
      //self rotZ := -45
      self newWall
    )

    // timestep ////////////////////////////////////////////////////////////////////////////-

    timeStep := method(
	if (self dead == 1 ) then (
	    wallHeight =  wallHeight * .95
	    if (wallHeight < .001, destroy)
	    return
	)

	s := normalSpeed
	speed = s + (speed - s) * .99
	speed = speed
	xpos = self xpos - sin(self rotY) * speed
	zpos = self zpos - cos(self rotY) * speed

	//self rotZ := self rotZ * .85
      
	checkForCollision 
	currentWall x2z2(xpos, zpos)

	if (self isAI == 1 and Number random < .01, makeRandomTurn)
    )

    hitWall := method(
       dead = 1
       speed = 0
       explosion := Explosion clone
      explosion rotY(rotY)
       dx, dy, dz := self dxdydzNormalized
      explosion position set(xpos - dx / 100, 0, zpos - dz / 100)
      Arena addObject(explosion)
      WallManager removeWalls(walls)
    )

    destroy := method(
	Arena removeObject(self) 
    )

    dxdydzNormalized := method(
	return -sin(self rotY) , 0, -cos(self rotY)
    )

    checkForCollision := method(
      // test for collision with arena walls
       edge := 4
      if ((xpos > edge or xpos < -edge) or (zpos > edge or zpos < -edge) ) then (
	  hitWall
      )
       
      // quickly find nearby walls
       walls, accelerateWalls
       xwalls := WallManager:wallsAroundX(xpos)
       zwalls := WallManager:wallsAroundZ(zpos)
       pos
      
      if (mod(floor(rotY), 180) == 0 ) then ( 
	 // travelling on x axis, so can only hit walls along z axis
	walls := zwalls
	accelerateWalls := xwalls
      else
	walls := xwalls
	accelerateWalls := zwalls
      )
      // need to also test next timestep position?
      
      if (walls ~= Nil ) then ( checkForCollisionWithWalls(walls) )
      if (accelerateWalls ~= Nil ) then ( accelerateIfNearWalls(accelerateWalls) )
      
      return 0
    )

    onXAxis := method(
      if (mod(floor(rotY), 180) == 0 ) then ( return 1 else return 0 )
    )

    between := method(a, b, c,
     if ((b >= min(a, c)) and (b <= max(a, c)) ) then ( return 1 else return 0 )
    )

    accelerateIfNearWalls := method((walls,
	//sort := method(a, b, return min(a, b), max(a, b))
	onX := onXAxis
	minDist := .15
	nearest := minDist
	walls foreach(wall,
	    if (onX == 1 ) then ( 
		if (between(wall z1, zpos, wall.z2)==1 ) then (
		    d := abs(xpos - wall.x1) 
		    nearest := nearest min(d)
		)
	    ) else ( 
		if (between(wall x1, xpos, wall.x2)==1 ) then (
		    d := abs(zpos - wall.z1)
		    nearest := nearest min(d)
		)
	    )
	)
	if (nearest == minDist, return )
	speed = speed + min(.0001, .01/(nearest*nearest))
    )

    checkForCollisionWithWalls := method(walls,
	walls foreach(wall, 
	    if (doesHitWall(wall) == 1, hitWall; return; )
	)
    )

    xyzNext := method(aWall,
       x2 := xpos - (sin(rotY) * speed *2)
       z2 := zpos - (cos(rotY) * self speed *2)
      return x2, 0, z2
    )

    doesHitWall := method(aWall,
       //sort := (a, b) return min(a, b), max(a, b) )

       x1 := xpos
       z1 := zpos
       x2, y2, z2 := xyzNext// hack
      
       x3 := aWall.x1
       z3 := aWall.z1
       x4 := aWall.x2
       z4 := aWall.z2
      
      // check if (lines are parallel
      if (((x1==x2) and (x3==x4)) or ((z1==z2) and (z3==z4)) ) then ( return 0 ) 
	    
      //if ((x1==x2) ) then ( // cycle on x axis
      //  if (between(x3, x1, x4)==1 and between(z3, z1, z2)==1 ) then ( return 1 )
      //else 
      //  if (between(z3, z1, z4)==1 and between(x1, x3, x2)==1 ) then ( return 1 )
      //)
      
      x1, x2 := sort(x1, x2)
      z1, z2 := sort(z1, z2)  
      x3, x4 := sort(x3, x4)
      z3, z4 := sort(z3, z4)  
      
      if ((x1==x2) ) then ( // cycle on x axis
	if ((x1 > x3) and (x1 < x4) and (z3 > z1) and (z3 < z2) ) then ( return 1 )
      else 
	if ((z1 > z3) and (z1 < z4) and (x3 > x1) and (x3 < x2) ) then ( return 1 )
      )
      return 0
    )

)