//////////////////////////////////////////////////////
// Point
//////////////////////////////////////////////////////

Point := Object clone do( 

  x := 0, 
  y := 0,
  z := 0,
)

Point:xreturn self x )
Point:yreturn self y )
Point:zreturn self z )

Point:x(x)
  self x := x
  return self
)

Point:y(y)
  self y := y
  return self
)

Point:z(z)
  self z := z
  return self
)

Point:xy(x, y)
  self x := x
  self y := y
  return self
)

Point:xyz(x, y, z)
  self x := x
  self y := y
  self z := z
  return self
)

Point:addXyz(x, y, z)
  self x := self x + x
  self y := self y + y
  self z := self z + z
  return self
)

Point:min(aPoint)
   x, y, z := aPoint:xyz
  self x := min(self x, x)
  self y := min(self y, y)
  self z := min(self z, z)
  return self
)

Point:max(aPoint)
   x, y, z := aPoint:xyz
  self x := max(self x, x)
  self y := max(self y, y)
  self z := max(self z, z)
  return self
)

Point:greaterThanOrEqualTo(aPoint)
   x, y, z := aPoint:xyz
  if (self x >= x and self y >= y and self z >= z ) then ( 
    return 1
  )
  return 0
)

Point:lessThanOrEqualTo(aPoint)
   x, y, z := aPoint:xyz
  if (self x <= x and self y <= y and self z <= z ) then ( 
    return 1
  )
  return 0
)



Point:one
  self x := 1
  self y := 1
  self z := 1
  return self
)

Point:zero
  self x := 0
  self y := 0
  self z := 0
  return self
)

Point:xy
  return self x, self y
)

Point:xyz
  return self x, self y, self z
)

Point:random
  self x := random-.5
  self y := random-.5
  self z := random-.5
  return self
)

Point:scaleBy(v)
  self x := self x * v
  self y := self y * v
  self z := self z * v
  return self
)

Point:append(point)
  self x := self x + point.x
  self y := self y + point.y
  self z := self z + point.z
  return self
)


Point:multiply(point)
  self x := self x * point.x
  self y := self y * point.y
  self z := self z * point.z
  return self
)

Point:subtract(point)
  self x := self x - point.x
  self y := self y - point.y
  self z := self z - point.z
  return self
)

Point:normalize
   r := self distanceFromOrigin
  self x := self x/r
  self y := self y/r
  self z := self z/r
  return self
)

Point:negate
  self x := - self x
  self y := - self y
  self z := - self z
  return self
)

Point:zero
  self x := 0
  self y := 0
  self z := 0
  return self
)

Point:copyFrom(point)
  self x := point.x
  self y := point.y
  self z := point.z
  return self
)

Point:rotate(angle)
   r := self distanceFromOrigin
  self x := r * cos(angle)
  self y := r * sin(angle)
  self z := r * sin(angle)
  return self
)

Point:distanceFromOrigin
   x, y, z := self xyz
  return sqrt((x*x) + (y*y) + (z*z))
)

Point:distanceSquaredTo(point)
   x1, y1, z1 := self xyz
   x2, y2, z2 := point:xyz
   x, y, z := x1 - x2, y1 - y2, z1 - z2
  return ((x*x) + (y*y) + (z*z))
)

Point:distanceTo(point)
  return sqrt(self distanceSquaredTo(point))
)


Point:rotate
  glRotate(self x, 1, 0, 0)
  glRotate(self y, 0, 1, 0)
  glRotate(self z, 0, 0, 1)
  return self
)

Point:translate
  glTranslate(self x, self y, self z)
  return self
)

Point:scale
  glScale(self x, self y, self z)
  return self
)

Point:dxdydzTo(aPoint)
    x1, y1, z1 := self xyz
    x2, y2, z2 := aPoint:xyz
    dx := x2-x1
    dy := y2-y1
    dz := z2-z1
   return dx, dy, dz
)

Point:moveCloserTobyRatio(aPoint, ac, max)
    dx, dy, dz := self dxdydzTo(aPoint)
   dx := dx*ac
   dy := dy*ac
   dz := dz*ac
   //if (max ~=Nil and max*max < (dx*dx)+(dy*dy)+(dz*dz) ) then ( 
   //  dx := dx/max
   //  dy := dy/max
   //  dz := dz/max
   //)
   self addXyz(dx, dy, dz)
   return self
)


