
Socket do(
	type := "Socket"
	docCategory("Networking")
	
    newSlot("bytesPerRead", 4096)
    newSlot("bytesPerWrite", 4096)
    newSlot("readTimeout", 60)
    newSlot("writeTimeout", 60)
    newSlot("connectTimeout", 60)
    newSlot("acceptTimeout", 365*24*60*60)
    newSlot("error", nil)
    newSlot("ipAddress", IPAddress clone)
 
    debugWriteln := nil
    debugOff := method(self debugWriteln := nil)
    debugOn := method(self debugWriteln := getSlot("writeln"))
   
	init := method(
	    resend
	    //writeln("Socket init ", self uniqueId)
		self ipAddress   := ipAddress clone
		
		self readBuffer  := Sequence clone
		self writeBuffer := Sequence clone
		
		self readEvent   := ReadEvent clone
		self writeEvent  := WriteEvent clone
		self
	)
    
	setHost := method(ip,
		ip at(0) isDigit ifFalse(ip = DNSResolver ipForHostName(ip))
		ipAddress setIp(ip)
		self
    )
    
	host := method(address host)

	setPort := method(port, ipAddress setPort(port); self)
	port := method(ipAddress port)

	checkErrno := method(
		//writeln("System errno = ", System errno)
		System errno ifNonNil(Exception raise(System errno))
	)

	setupEvents := method(
		readEvent setDescriptorId(descriptorId)
		writeEvent setDescriptorId(descriptorId)
	)
    
    streamOpen := method(
        asyncStreamOpen
        checkErrno
        setupEvents
        true
    )
    
    udpOpen := method(
        asyncUdpOpen
        checkErrno
        setupEvents
		true
    )
    
    raiseExceptionIfInvalid := method(
    	if(self isValid == false, Exception raise("invalid socket"); writeln("invalid socket"); System exit)
    )
    
	connect := method(
		debugWriteln("Socket connect isOpen = ", isOpen)
	    isOpen ifFalse(streamOpen)
        didConnect := asyncConnect(ipAddress)
        didConnect ifFalse(
            writeEvent waitOn(connectTimeout) ifFalse(Exception raise("connection timeout"))
            raiseExceptionIfInvalid
            didConnect = asyncConnect(ipAddress)
            //checkErrno
        )
 		//writeln("didConnect = ", didConnect)
        didConnect ifTrue(return true)
        checkErrno
        Exception raise("unable to connect to " .. ipAddress ip .. " on port " .. ipAddress port)
        false
	)

	streamRead := method(numBytes,
		total := readBuffer size + numBytes
		readEvent setDescriptorId(descriptorId)
		
		while(readBuffer size < total, 
		    asyncStreamRead(readBuffer, bytesPerRead) ifFalse(readEvent waitOnOrExcept)
		    raiseExceptionIfInvalid
		)
        debugWriteln(Scheduler currentCoroutine label, " read [", readBuffer, "]")

		//checkErrno
	)

	streamWrite := method(buffer, 
		writeBuffer copy(buffer)
		bytesRemaining := writeBuffer size

	    debugWriteln(Scheduler currentCoroutine label, " writing [", writeBuffer, "]")
		while(bytesRemaining > 0,
			startIndex := writeBuffer size - bytesRemaining
			bytesWritten := asyncStreamWrite(writeBuffer, startIndex, bytesPerWrite)
			debugWriteln("bytesWritten = ", bytesWritten)
			//bytesWritten ifTrue(bytesWritten = bytesPerWrite)
			bytesRemaining = bytesRemaining - bytesWritten
			writeEvent waitOnOrExcept(writeTimeout)
			raiseExceptionIfInvalid
        )
        
	)

	streamReadNextChunk := method(
	    self isOpen ifFalse(return false)
	    self isValid ifFalse(return false)
	    done := false
	     
		while(isValid and(bytesRead := asyncStreamRead(readBuffer, bytesPerRead) > 0), 
			//writeln("bytesRead = ", bytesRead)
			done = true
		)
		
		if(bytesRead == -1 or bytesRead == 0, checkErrno)
		
		if (done, return true)
		
		raiseExceptionIfInvalid
		readEvent waitOnOrExcept(readTimeout)
		raiseExceptionIfInvalid

		while(isValid and(bytesRead := asyncStreamRead(readBuffer, bytesPerRead) > 0), nil)
		
		if(bytesRead == -1, checkErrno)
		
	    debugWriteln(Scheduler currentCoroutine label, " read [", readBuffer, "] ", readBuffer size)
		true
	)

	udpReadNextChunk := method(ipAddress, 
		raiseExceptionIfInvalid
		readEvent waitOnOrExcept(readTimeout)
		raiseExceptionIfInvalid
		while(asyncUdpRead(ipAddress, readBuffer, bytesPerRead), raiseExceptionIfInvalid; nil)
		true
	)
			
	udpRead := method(ipAddress, numBytes, 
		total := readBuffer size + numBytes

		while(readBuffer size < total, 
			raiseExceptionIfInvalid
		    asyncUdpRead(ipAddress, readBuffer, bytesPerRead) ifFalse(readCoro waitOnOrExcept(readTimeout))
		    raiseExceptionIfInvalid
		    checkErrno
		)
	)

	udpWrite := getSlot("asyncUdpWrite")
	
	serverOpen := method(
	   streamOpen
	   asyncBind(ipAddress)
	   asyncListen
	)
	
	serverWaitForConnection := method(
		isOpen ifFalse(serverOpen)
	    newAddress := IPAddress clone
		socket := asyncAccept(newAddress)
		socket ifNil(
            debugWriteln("serverWaitForConnection")
            raiseExceptionIfInvalid
            readEvent waitOn(acceptTimeout) ifFalse(Exception raise("accept timeout"))
            raiseExceptionIfInvalid
            debugWriteln("serverWaitForConnection after wait")
            socket := asyncAccept(newAddress)
            socket init 
            socket setupEvents
            //checkErrno
		)
        if(socket, socket setIpAddress(newAddress), nil)
	)
	
	write := method(b,
	   writeln("Warning: Socket write dreprecated - use streamWrite instead")
	   call message arguments foreach(arg, 
	       streamWrite(call sender doMessage(arg))
	   )
	)
	
    read := method(
	   	writeln("Warning: Socket read dreprecated - use streamReadNextChunk instead")
        streamReadNextChunk
	)
)

