#include <iostream>
#include <string>

extern "C"
{
#include "IoState.h"
#include "IoObject.h"
}

class Controller
{
public:
    Controller();
    ~Controller();
    static void print(Controller * self, char * s);
    static void exception(Controller * self, IoException * e);
    static void exit(Controller * self);
    int main(int argc, const char * argv[]);
protected:
    IoState *state;
private:
};

Controller::Controller()
{
    state = IoState_new();
    if (state == NULL) {
        throw("IoState_new returned NULL");
	}
    IoState_callbackContext_(state, this);
    IoState_printCallback_(state, (IoStatePrintCallback 
*)Controller::print);
    IoState_exceptionCallback_(state, 
(IoStateExceptionCallback*)Controller::exception);
    IoState_exitCallback_(state, (IoStateExitCallback *)Controller::exit);
}

Controller::~Controller()
{
    IoState_free(state);
    state = NULL;
}

void Controller::print(Controller * self, char * s)
{
	std::cout << s;
}

void Controller::exception(Controller * self, IoException * e)
{
    std::cout << "Exception: " << IoException_name(e) << " - " << 
IoException_description(e) << std::endl;
}

void Controller::exit(Controller * self)
{
	// NOTE : if we simply try to call exit() from here, we will call
	// ourselves, which will call ourselves, which will call...
	// Infinite recursion and stack explosion.
	// Use the :: global scope operator to state that you are
	// trying to call the C runtime exit() function.
	::exit(0);
}

int Controller::main(int argc, const char * argv[])
{
    IoObject *result;
    IoState_argc_argv_(state, argc, argv);
    std::string buffer;
    std::cout << "Io C++ test" << std::endl;

	while (true) {
        std::cout << "C++> ";
        std::getline (std::cin, buffer);
        result = IoState_doCString_(state, (char*)buffer.c_str());
        std::cout << "-=> ";
        IoObject_print(result);
        std::cout << std::endl;
    }
}

int main(int argc, const char* argv[])
{
    Controller c;
    int ReturnCode = c.main(argc, argv);
    return 0;
}

