

OSWindow := GLObject clone do(
    type   := "OSWindow"
	    
    newSlot("position", Point clone set(500,500))
    size    := Point clone set(500,500)
    setSize := method(w, h, size set(w, h))
    width   := method(size width)
    height  := method(size height)
    newSlot("title", "")
    newSlot("firstResponder")
    newSlot("timers", Map clone)
    newSlot("backgroundColor", Color clone)
    newSlot("application")
    
    init := method(
		position = position clone
		size = size clone
		setFirstResponder(nil)
		setTimers(Map clone)
    )

    open := method(
    	//glutInit
		glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH)
		glutInitWindowPosition(position x, position y)
		glutInitWindowSize(width, height)
		glutCreateWindow(title)
		glutReshapeWindow(width, height)
		glutEventTarget(self)
		glutTimerFunc(1, 0)
		glutDisplayFunc
		glutKeyboardFunc
		glutSpecialFunc
		glutMotionFunc
		glutMouseFunc
		glutReshapeFunc
		enableAntialiasing
		//glutMainLoop
    )
    
    enableAntialiasing := method(  
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		glLineWidth(1)
		glEnable(GL_LINE_SMOOTH)
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST)
		glShadeModel(GL_SMOOTH)
    )

    // --- Timer Events -------------

    addTimerTargetWithDelay := method(target, delay,
		id := target uniqueId 
		idString := id asString
		timers atPut(idString, target)
		glutTimerFunc(1000 * delay, id)
    )

    removeTimerWithTarget := method(target, 
		timers removeAt(target uniqueId asString)
    )

    timer := method(id,
		idString := id asString
		target := timers at(idString)
		if(target,
			timers removeAt(idString)
			target timer(application)
		)
    )

    // --- Subviews ---------------------------------

    subviews := List clone
    
    addSubview := method(view, 
		subviews append(view)
		view setTopWindow(self)
    )
    
    removeSubview := method(view, 
		view releaseFirstResponder
		subviews remove(view)
		//view setTopWindow(Nil)
    )

    orderToFrontSubview := method(view,
		subviews remove(view)
		subviews append(view)
    )

    // --- Display ---------------------------------

    set3dPerpective := method(
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity
		gluPerspective(45, width/height, 0.1, 100)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity
    )

    set2dPerpective := method(
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity
		gluOrtho2D(0, width, 0, height)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity
    )

    reshape := method(w, h, 
		//write("reshape(", w, ",", h, ")\n")
		resizeTo(w, h)
		glViewport(0, 0, w, h)
		set2dPerpective
		glutPostRedisplay
    )

    resizeTo := method(w, h,
		w = w roundDown
		h = h roundDown
		dx := w - width
		dy := h - height 
		size set(w, h)
		subviews foreach(v, v resizeBy(dx, dy))
    )

    display := method(
		//write("Screen display ", Date clone now asNumber, "\n")
		glLoadIdentity
		glScissor(0, 0, width, height)
		drawBackground
		subviews foreach(v, v display)
		glFlush
		glutSwapBuffers
    )

    drawBackground := method(
		//glClearColor(20/256, 100/256, 184/256, 1)
		glClearColor(backgroundColor red, backgroundColor green, backgroundColor blue, 1)
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    )
    
   // --- Events ---------------------------------

    setFirstResponder := method(fr, 
		//writeln("OSWindow setFirstResponder(", fr uniqueId, ")")
		//if (firstResponder, firstResponder releaseFirstResponder)
		firstResponder = fr
    )

    keyboard := method(key, x, y,
    	Keyboard key(key)
		//write("OSWindow keyboard ", key, " firstResponder ", firstResponder type, "\n")
		firstResponder ?keyboard(key, x, y)
		glutPostRedisplay
    )
    
    special := method(key, x, y, 
    	Keyboard special(key)
		//write("special ", key, "\n")
		firstResponder ?special(key, x, y)
		glutPostRedisplay
    )
	
    mouse := method(b, s, x, y,
		y = height - y
		//writeln("----------------------------------------")
		//writeln("OSWindow mouse ", b, " ", s, " ", x, ", ", y)
		//writeln("OSWindow firstResponder = ", firstResponder type, "_", firstResponder uniqueId)
		Mouse mouse(b, s, x, y)
		//writeln("Mouse  ", Mouse uniqueId)
		//writeln("Mouse position ", Mouse position)
		 
		if(firstResponder and s == 1) then(firstResponder directMouse) else(
			hv := hitView
			//writeln(self type, " hitView = ", hv type)
			//if(hv == nil, firstResponder = hv)
			//if(hv and hv acceptsFirstResponder, firstResponder = hv)	

			if(hv, hv mouse)
		)
		
		display
    )

    motion := method(x, y,
		y = height - y
		Mouse motion(x, y)
		if(firstResponder, firstResponder motion)
    )

    hitView := method(
		subviews reverseForeach(i, view, 
			v := view screenHit
			//writeln(self type, " screenHit = ", v type)
			if(v, return v)
		)
		return nil
    )
)

