/*   
  copyright: Steve Dekorte, 2004. All rights reserved.
  license: See _BSDLicense.txt.
    
  SearchEngine can be used to index and retrieve Documents via a word they contain.
    
    Documents:
    
      A skipdb indexed by some doc key (say, a url).
      
    Document:
    
      Holds an array of word PIDs (so it can remove itself from 
      their doc indexes when it is removed).
    
    Words:
    
      A skipdb indexed by the word.
      
    Word:
    
      Holds an array of documents PIDs.
    
  Usage example:
    
    index:
    
      Crawl the web and add each page as a document with the url as 
      the document key each word on the page is added to the document
    
    search:
    
      Lookup a word and get it's list of document ids. 
      Lookup each document to get it's url.
        
    remove a document:
    
      Lookup a document by it's URL and tell each word in it's word list to 
      remove the document from it's doc index, then remove the doc
      
    remove a word:
    
      This can be a slow operation as documents contain a word array instead of a word index.
    
*/

#ifndef SearchEngine_DEFINED
#define SearchEngine_DEFINED 1

#include "SkipDB.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
{    
    SkipDB *wordDB;
    SkipDB *docsDB;
} SearchEngine;

SearchEngine *SearchEngine_new(void);
void SearchEngine_free(SearchEngine *self);

Document *SearchEngine_documentAt_(SearchEngine *self, char *docKey);
List *SearchEngine_documentsWithWord_(SearchEngine *self, char *path);

#ifdef __cplusplus
}
#endif
#endif
